function [xo, vout, hist] = cycle(xi,df,list)
% cycle
% [xo, vout, hist] = cycle(xi, @df)
%
%   xi = starting point (design variables)
%   df = defect function f = df(x), use function reference
%
%   xo = solution
%   vout = history of solution vectors, each row is a new step
%   hist = [fval pmin nr], each row is a new step;

if (nargin<3)
    list = 0;
else
    list = 1;
end

nv = length(xi);
xo = xi(:);
fmt = '%2d %10.4g %10.3g %10.3g %2d\n';
f = feval(df,xo);
fprev = norm(f);
vout(1,:)=xo';
hist(1,:)=[fprev 0 0];
sing = 1e-4;
if (list)
    fprintf('%2s %10s %10s %10s %2s\n','#','merit','damp','%step','nv');
    fprintf('%2d %10.4g\n',0,fprev);
end
%mov = avifile('linear_search.avi','fps',1);
for k=1:1000
    [A fz] = calculate_derivatives(df,xo);
    [U W V] = svd(A,0);
    d = diag(W);
    T = zeros(size(V));
    dmax = sing*max(d);
    for j=1:nv
        if (d(j)>dmax)
            T(j,j)=1/d(j);
        end
    end
    dv = -V*T'*U'*fz;
    [pmin fval]=fminbnd(@sfunc,0,1,[],df,xo,dv);
%    show_search(xo);
%    f = getframe(gcf);
%    mov = addframe(mov,f);
    xo = xo+pmin*dv;
    f = feval(df,xo);
    fval = norm(f);
    nr = length(find(d>dmax));
    vout(k+1,:)=xo';
    hist(k+1,:)=[fval pmin nr];
    rstep = norm(dv)/norm(xo);
    if (list)
        fprintf(fmt,k,fval,pmin,100.*pmin*rstep,nr);
    end
%    pause();
    if (fval<1e-10 | abs(fval-fprev)<1e-10 | rstep<1e-10)
        break;
    end
    fprev = fval;
    if (pmin<1e-10)
        if (nr<nv)
            sing=0.01*sing;
        else
            break;
        end
    end
end
%mov = close(mov);
